@@ -19,12 +19,13 @@ from stsci.tools import fileutil, logutil, mputil, teal
 from . import quickDeriv
 from . import util
 from . import processInput
-from . version import __version__, __version_date__
+from . import __version__
+
 if util.can_parallel:
     import multiprocessing
 
 
-__taskname__ = "drizzlepac.drizCR"  # looks in drizzlepac for sky.cfg
+__taskname__ = "drizCR"  # looks in drizzlepac for sky.cfg
 _STEP_NUM = 6  # this relates directly to the syntax in the cfg file
 
 
@@ -76,11 +77,12 @@ def rundrizCR(imgObjList, configObj, procSteps=None):
     subprocs = []
     if pool_size > 1:
         log.info('Executing {:d} parallel workers'.format(pool_size))
+        mp_ctx = multiprocessing.get_context('fork')
         for image in imgObjList:
-            manager = multiprocessing.Manager()
+            manager = mp_ctx.Manager()
             mgr = manager.dict({})
 
-            p = multiprocessing.Process(
+            p = mp_ctx.Process(
                 target=_driz_cr,
                 name='drizCR._driz_cr()',  # for err msgs
                 args=(image, mgr, paramDict.dict())
@@ -341,54 +343,6 @@ def setDefaults(configObj={}):
     return paramDict
 
 
-def help(file=None):
-    """
-    Print out syntax help for running ``astrodrizzle``
-
-    Parameters
-    ----------
-    file : str (Default = None)
-        If given, write out help to the filename specified by this parameter
-        Any previously existing file with this name will be deleted before
-        writing out the help.
-
-    """
-    helpstr = getHelpAsString(docstring=True, show_ver=True)
-    if file is None:
-        print(helpstr)
-    else:
-        with open(file, mode='w') as f:
-            f.write(helpstr)
-
-
-def getHelpAsString(docstring=False, show_ver=True):
-    """
-    Return useful help from a file in the script directory called
-    ``__taskname__.help``
-
-    """
-    install_dir = os.path.dirname(__file__)
-    taskname = util.base_taskname(__taskname__, __package__)
-    htmlfile = os.path.join(install_dir, 'htmlhelp', taskname + '.html')
-    helpfile = os.path.join(install_dir, taskname + '.help')
-
-    if docstring or (not docstring and not os.path.exists(htmlfile)):
-        if show_ver:
-            helpString = "\n{:s} Version {:s} updated on {:s}\n\n".format(
-                __taskname__, __version__, __version_date__
-            )
-        else:
-            helpString = ''
-
-        if os.path.exists(helpfile):
-            helpString += teal.getHelpFileAsString(taskname, __file__)
-        elif __doc__ is not None:
-                helpString += __doc__ + os.linesep
-
-    else:
-        helpString = 'file://' + htmlfile
-
-    return helpString
-
-
-drizCR.__doc__ = getHelpAsString(docstring=True, show_ver=False)
+drizCR.__doc__ = util._def_help_functions(
+    locals(), module_file=__file__, task_name=__taskname__, module_doc=__doc__
+)
