@@ -1,10 +1,15 @@
-""" Utility to analyze an input dataset and determine whether the dataset can be aligned
+""" Utilities to analyze an input and determine whether the input is viable for a given process
 
-The function analyze_data opens an input list containing FLT and/or FLC FITS filenames
-in order to access the primary header data.  Based upon the values of specific
+The fundamental function analyze_data opens an input list containing FLT and/or FLC FITS
+filenames in order to access the primary header data.  Based upon the values of specific
 FITS keywords, the function determines whether or not each file within this dataset
 can or should be reconciled against an astrometric catalog and, for multiple images, used
 to create a mosaic.
+
+The functions, mvm_analyze_wrapper and analyze_wrapper, are thin wrappers around analyze_data
+to accommodate special case uses.  The mvm_analyze_wrapper takes a filename as input and
+returns a boolean indicator.  The analyze_wrapper has the same function signature as
+analyze_data, but it returns a list instead of an astropy table.
 """
 import math
 import sys
@@ -20,10 +25,10 @@ __taskname__ = 'analyze'
 
 MSG_DATEFMT = '%Y%j%H%M%S'
 SPLUNK_MSG_FORMAT = '%(asctime)s %(levelname)s src=%(name)s- %(message)s'
-log = logutil.create_logger(__name__, level=logutil.logging.NOTSET, stream=sys.stdout,
+log = logutil.create_logger(__name__, level=logutil.logging.DEBUG, stream=sys.stdout,
                             format=SPLUNK_MSG_FORMAT, datefmt=MSG_DATEFMT)
 
-__all__ = ['analyze_data']
+__all__ = ['analyze_data', 'analyze_wrapper', 'mvm_analyze_wrapper']
 
 # Define global default keyword names for these fields
 OBSKEY = 'OBSTYPE'
@@ -37,12 +42,19 @@ TARKEY = 'TARGNAME'
 EXPKEY = 'EXPTIME'
 FGSKEY = 'FGSLOCK'
 CHINKEY = 'CHINJECT'
+DRIZKEY = 'DRIZCORR'
 
+# Return codes 
+class Ret_code(Enum):
+    """
+    Define return status codes for Operations 
+    """
+    OK = 0
+    SBCHRC_DATA = 55 
+    NO_VIABLE_DATA = 65
 
 # Annotates level to which image can be aligned according observational parameters
 # as described through FITS keywords
-
-
 class Messages(Enum):
     """
     Define a local classification for OK, Warning, and NoProcess messages
@@ -51,7 +63,44 @@ class Messages(Enum):
     OK, WARN, NOPROC = 1, -1, -2
 
 
-def analyze_wrapper(input_file_list, log_level=logutil.logging.NOTSET):
+def mvm_analyze_wrapper(input_filename, log_level=logutil.logging.DEBUG):
+    """
+    Thin wrapper for the analyze_data function to return a viability indicator regarding a image for MVM processing.
+
+    Parameters
+    ==========
+    input_filename : string
+        Full fileName of data to be analyzed for viability to be processed as an MVM constituent.
+
+    Returns
+    =======
+    use_for_mvm : boolean
+        Boolean which indicates whether the input image should be used for MVM processing -
+        True: use for MVM, False: do NOT use for MVM
+
+    Note: This routine is invoked externally by software used for operations.
+
+    """
+    # Set logging level to user-specified level
+    log.setLevel(log_level)
+
+    # Invoke the low-level analyze_data routine with type = "MVM"
+    filtered_table = analyze_data([input_filename], type = "MVM")
+
+    # There is only one row in this output table
+    use_for_mvm = False
+    if filtered_table['doProcess'] == 0:
+        use_for_mvm = False
+        log.warning("Image, {}, cannot be used for MVM processing.  Issue: {}.\n". \
+                    format(input_filename, filtered_table['processMsg'][0]))
+    else:
+        use_for_mvm = True
+        log.info("Image, {}, will be used for MVM processing.".format(input_filename))
+
+    return use_for_mvm
+
+
+def analyze_wrapper(input_file_list, log_level=logutil.logging.DEBUG, use_sbchrc=True, type=""):
     """
     Thin wrapper for the analyze_data function to return a list of viable images.
 
@@ -62,34 +111,90 @@ def analyze_wrapper(input_file_list, log_level=logutil.logging.NOTSET):
         dataset where 'associated dataset' may be a single image, multiple images, an HST
         association, or a number of HST associations
 
+    log_level : int, optional
+        The desired level of verboseness in the log statements displayed on the screen and written to the .log file.
+        Default value is 20, or 'info'.
+
+    use_sbchrc : bool, optional
+        Boolean based upon the environment variable setting of MVM_INCLUDE_SMALL which indicates whether
+        or not to generate MVM SkyCell layers from ACS/HRC and ACS/SBC data.  These exposures typically
+        only cover a miniscule fraction of a SkyCell and the plate scale of the SkyCell would result in
+        a degraded representation of the original ACS/HRC and ACS/SBC data.  A setting of 'False' turns off
+        the generation of layers from ACS/HRC and ACS/SBC data, and a setting of 'True' turns the generation on.
+        Default = True
+
+    type : string, optional
+        String indicating whether this file is for MVM or some other processing.
+        If type == "MVM", then Grism/Prism data is ignored.  If type == "" (default) or any
+        other string, the Grism/Prism data is considered available for processing unless there is
+        some other issue (i.e., exposure time of zero).
+        Default = ""
+
     Returns
     =======
     viable_images_list : list
        List of images which can be used in the drizzle process.
 
-    This routine returns a list containing only viable images instead of a table which
-    provides information, as well as a doProcess bool, regarding each image.
+    return_code : int
+        Numeric code indicative of the status of the analysis of the input data.
+        These return codes are defined in this module by class Ret_code.
+
+    Note: This routine returns a *list*, as well as a return code, containing only viable images
+    instead of a table which provides information, as well as a doProcess bool, regarding each image.
     """
-    # set logging level to user-specified level
+    # Set logging level to user-specified level
     log.setLevel(log_level)
-
-    process_list = []
-
+ 
     # Analyze the input file list and get the full table assessment
-    filtered_table = analyze_data(input_file_list)
+    filtered_table = analyze_data(input_file_list, type=type)
 
-    # Extract only the filenames of viable images for processing (i.e., doProcess == 1)
-    if filtered_table['doProcess'].sum() == 0:
-        log.error("No viable images in single/multiple visit table - no processing done.\n")
+    # Reduce table to only the data which should be processed (doProcess == 1)
+    mask = filtered_table["doProcess"] > 0
+    filtered_table = filtered_table[mask]
+
+    good_table = None
+    good_rows = []
+    process_list = []
+    return_value = Ret_code.OK.value
+
+    # MVM processing, but excluding SBC/HRC data
+    if use_sbchrc == False and type.upper() == "MVM": 
+        # Check the table to determine presence of SBC/HRC data
+        if filtered_table:
+            for i, old_row in enumerate(filtered_table):
+                if old_row["detector"].upper() != "SBC" and old_row["detector"].upper() != "HRC":
+                    good_rows.append(old_row)
+
+            # The entire filtered_table contains only SBC or HRC data
+            if not good_rows:
+                log.warning("Only non-viable or SBC/HRC images in the multi-visit table - no processing done.\n")
+                return_value = Ret_code.SBCHRC_DATA.value
+            # Table contains some non-SBC/non-HRC data for processing
+            else:
+                good_table = Table(rows=good_rows, names=filtered_table.colnames)
+
+                # Get the list of all "good" files to use for the alignment
+                process_list = good_table['imageName'].tolist()
+                return_value = Ret_code.OK.value
+
+        # There is already nothing to process based upon the analysis criteria
+        else:
+            log.warning("No viable images in multi-visit table - no processing done.\n")
+            return_value = Ret_code.NO_VIABLE_DATA.value
+ 
+    # SVM processing or MVM processing with SBC/HRC data included in the MVM processing
     else:
-        # Get the list of all "good" files to use for the alignment
-        process_list = filtered_table['imageName'][np.where(filtered_table['doProcess'])]
-        process_list = list(process_list)  # Convert process_list from numpy list to regular python list
+        if filtered_table:
+            # Get the list of all "good" files to use for the alignment
+            process_list = filtered_table['imageName'].tolist()
+        else:
+            log.warning("No viable images in single/multi-visit table - no processing done.\n")
+            return_value = Ret_code.NO_VIABLE_DATA.value
 
-    return process_list
+    return process_list, return_value
 
 
-def analyze_data(input_file_list, log_level=logutil.logging.NOTSET):
+def analyze_data(input_file_list, log_level=logutil.logging.DEBUG, type=""):
     """
     Determine if images within the dataset can be aligned
 
@@ -104,6 +209,12 @@ def analyze_data(input_file_list, log_level=logutil.logging.NOTSET):
         The desired level of verboseness in the log statements displayed on the screen and written to the .log file.
         Default value is 20, or 'info'.
 
+    type : string
+        String indicating whether this file is for MVM or some other processing.
+        If type == "MVM", then Grism/Prism data is ignored.  If type == "" (default) or any
+        other string, the Grism/Prism data is considered available for processing unless there is
+        some other issue (i.e., exposure time of zero).
+
     Returns
     =======
     output_table : object
@@ -123,6 +234,7 @@ def analyze_data(input_file_list, log_level=logutil.logging.NOTSET):
     TARGNAME : DARK, TUNGSTEN, BIAS, FLAT, EARTH-CALIB, DEUTERIUM
     EXPTIME : 0
     CHINJECT : is not NONE
+    DRIZCORR : OMIT, SKIPPED
 
     The keyword/value pairs below define the category which the data can be processed, but
     the results may be compromised
@@ -133,7 +245,7 @@ def analyze_data(input_file_list, log_level=logutil.logging.NOTSET):
 
     Please be aware of the FITS keyword value NONE vs the Python None.
     """
-    # set logging level to user-specified level
+    # Set logging level to user-specified level
     log.setLevel(log_level)
 
     acs_filt_name_list = [FILKEY1, FILKEY2]
@@ -204,6 +316,7 @@ def analyze_data(input_file_list, log_level=logutil.logging.NOTSET):
         mjdutc = header_data['EXPSTART']
 
         # Obtain keyword values for analysis of viability
+        drizcorr = (header_data[DRIZKEY]).upper()
         obstype = (header_data[OBSKEY]).upper()
         mtflag = (header_data[MTKEY]).upper()
         scan_typ = ''
@@ -242,6 +355,10 @@ def analyze_data(input_file_list, log_level=logutil.logging.NOTSET):
         # Determine if the image has one of these conditions.  The routine
         # will exit processing upon the first satisfied condition.
 
+        # Compute if the exposure time is very close to zero as it will be
+        # needed when deciding whether or not to use the particular Grism/Prism data
+        is_zero = True if math.isclose(exptime, 0.0, abs_tol=1e-5) else False
+
         no_proc_key = None
         no_proc_value = None
         do_process = True
@@ -250,6 +367,11 @@ def analyze_data(input_file_list, log_level=logutil.logging.NOTSET):
             no_proc_key = OBSKEY
             no_proc_value = obstype
 
+        # drizzling has been turned off
+        elif drizcorr in ['OMIT', 'SKIPPED']:
+            no_proc_key = DRIZKEY
+            no_proc_value = drizcorr
+
         # Moving target
         elif mtflag == 'T':
             no_proc_key = MTKEY
@@ -280,12 +402,33 @@ def analyze_data(input_file_list, log_level=logutil.logging.NOTSET):
             no_proc_key = CHINKEY
             no_proc_value = chinject
 
-        # Filter name which starts with "G" for Grism, "PR" for Prism, or
-        # "BLOCK" for internal calibration of SBC
+        # Filter name which starts with "BLOCK" for internal calibration of SBC
         # The sfilter variable may be the concatenation of two filters (F160_CLEAR)
-        split_sfilter = sfilter.split('_')
+        #
+        # Grism/Prism images are also IMAGING=SPECTROSCOPIC, suppress the "no processing"
+        # indicators to allow the Grism/Prism images to be minimally processed for
+        # keyword updates.  This was done as a retrofit to allow Grism/Prism images
+        # to have the same WCS solution as the direct images in the visit (same detector).
+        # The exception to this will be if the Grism/Prism data has a zero exposure time as
+        # the WCS will be only "OPUS" under this condition, and this will disrupt processing
+        # for all the good data.
+        split_sfilter = sfilter.upper().split('_')
         for item in split_sfilter:
-            if item.startswith(('G', 'PR', 'BLOCK')):
+            # This is the only circumstance when Grism/Prism data WILL be processed.
+            if item.startswith(('G', 'PR')) and not is_zero and type.upper() == "SVM":
+                no_proc_key = None
+                no_proc_value = None
+                log.info("The Grism/Prism data, {}, will be processed.".format(input_file))
+            # Grism/Prism WILL NOT be processed primarily if MVM processing or with an exposure time of zero.
+            elif item.startswith(('G', 'PR')): 
+                if type.upper() == "MVM":
+                    no_proc_value += ", Grism/Prism data and MVM processing"
+                    log.warning("The Grism/Prism data {} with MVM processing will be ignored.".format(input_file))
+                elif is_zero:
+                    no_proc_value += ", Grism/Prism data and EXPTIME = 0.0"
+                    log.warning("The Grism/Prism data {} with zero exposure time will be ignored.".format(input_file))
+    
+            if item.startswith(('BLOCK')):
                 no_proc_key = FILKEY
                 no_proc_value = sfilter
 
