@@ -7,10 +7,10 @@ This module manages the creation of the output image FITS file.
 
 """
 from astropy.io import fits
-from stsci.tools import fileutil, readgeis, logutil
+from stsci.tools import fileutil, logutil
 
 from . import wcs_functions
-from . import version
+from . import __version__
 from . import updatehdr
 
 from fitsblender import blendheaders
@@ -18,13 +18,19 @@ from fitsblender import blendheaders
 yes = True
 no = False
 
-RESERVED_KEYS = ['NAXIS', 'BITPIX', 'DATE', 'IRAF-TLM', 'XTENSION', 'EXTNAME',' EXTVER']
+RESERVED_KEYS = ['NAXIS', 'NAXIS1', 'NAXIS2', 'EXTEND',
+                 'BITPIX', 'DATE', 'IRAF-TLM',
+                 'XTENSION', 'EXTNAME', 'EXTVER']
 
 EXTLIST = ('SCI', 'WHT', 'CTX')
 
 WCS_KEYWORDS = ['CD1_1', 'CD1_2', 'CD2_1', 'CD2_2', 'CRPIX1',
 'CRPIX2', 'CRVAL1', 'CRVAL2', 'CTYPE1', 'CTYPE2', 'WCSNAME']
 
+DIST_KWS = ['D2IMERR1', 'D2IMERR2', 'D2IMDIS1', 'D2IMDIS2',
+            'D2IMEXT', 'DP1', 'DP2', ]
+DIST_MULTI_KWS = ['D2IM1.*', 'D2IM2.*', 'CPDIS?', 'CPERR?', 'TDD_C*']
+
 # fits.CompImageHDU() crashes with default arguments.
 # Instead check that fits module has *attribute* 'CompImageHDU':
 PYFITS_COMPRESSION = hasattr(fits, 'CompImageHDU')
@@ -32,21 +38,21 @@ PYFITS_COMPRESSION = hasattr(fits, 'CompImageHDU')
 # Set up dictionary of default keywords to be written out to the header
 # of the output drizzle image using writeDrizKeywords()
 DRIZ_KEYWORDS = {
-                'VER':{'value':"",'comment':'Drizzle, task version'},
-                'GEOM':{'value':"wcs",'comment':'Drizzle, source of geometric information'},
-                'DATA':{'value':"",'comment':'Drizzle, input data image'},
-                'DEXP':{'value':"",'comment':'Drizzle, input image exposure time (s)'},
-                'OUDA':{'value':"",'comment':'Drizzle, output data image'},
-                'OUWE':{'value':"",'comment':'Drizzle, output weighting image'},
-                'OUCO':{'value':"",'comment':'Drizzle, output context image'},
-                'MASK':{'value':"",'comment':'Drizzle, input weighting image'},
-                'WTSC':{'value':"",'comment':'Drizzle, weighting factor for input image'},
-                'KERN':{'value':"",'comment':'Drizzle, form of weight distribution kernel'},
-                'PIXF':{'value':"1.0",'comment':'Drizzle, linear size of drop'},
-                'COEF':{'value':"SIP",'comment':'Drizzle, source of coefficients'},
-                'OUUN':{'value':"cps",'comment':'Drizzle, units of output image - counts or cps'},
-                'FVAL':{'value':"INDEF",'comment':'Drizzle, fill value for zero weight output pix'},
-                'WKEY':{'value':"",'comment':'Input image WCS Version used'}
+                'VER': {'value': "", 'comment': 'Drizzle, task version'},
+                'GEOM': {'value': "wcs", 'comment': 'Drizzle, source of geometric information'},
+                'DATA': {'value': "", 'comment': 'Drizzle, input data image'},
+                'DEXP': {'value': "", 'comment': 'Drizzle, input image exposure time (s)'},
+                'OUDA': {'value': "", 'comment': 'Drizzle, output data image'},
+                'OUWE': {'value': "", 'comment': 'Drizzle, output weighting image'},
+                'OUCO': {'value': "", 'comment': 'Drizzle, output context image'},
+                'MASK': {'value': "", 'comment': 'Drizzle, input weighting image'},
+                'WTSC': {'value': "", 'comment': 'Drizzle, weighting factor for input image'},
+                'KERN': {'value': "", 'comment': 'Drizzle, form of weight distribution kernel'},
+                'PIXF': {'value': "1.0", 'comment': 'Drizzle, linear size of drop'},
+                'COEF': {'value': "SIP", 'comment': 'Drizzle, source of coefficients'},
+                'OUUN': {'value': "cps", 'comment': 'Drizzle, units of output image - counts or cps'},
+                'FVAL': {'value': "INDEF", 'comment': 'Drizzle, fill value for zero weight output pix'},
+                'WKEY': {'value': "", 'comment': 'Input image WCS Version used'}
                 }
 
 log = logutil.create_logger(__name__, level=logutil.logging.NOTSET)
@@ -105,7 +111,7 @@ class OutputImage:
         self.blot = blot
 
         if PYFITS_COMPRESSION and 'compress' in input_pars:
-            self.compress = input_pars['compress'] # Control creation of compressed FITS files
+            self.compress = input_pars['compress']  # Control creation of compressed FITS files
         else:
             self.compress = False
 
@@ -115,10 +121,10 @@ class OutputImage:
 
         if not blot:
             self.output = plist[0]['output']
-            self.shape = (plist[0]['outny'],plist[0]['outnx'])
+            self.shape = (plist[0]['outny'], plist[0]['outnx'])
         else:
             self.output = plist[0]['blotImage']
-            self.shape = (plist[0]['blotny'],plist[0]['blotnx'])
+            self.shape = (plist[0]['blotny'], plist[0]['blotnx'])
 
         # Keep track of desired output WCS computed by PyDrizzle
         self.wcs = wcs
@@ -145,7 +151,7 @@ class OutputImage:
             # Report values appropriate for entire combined product
             self.texptime = plist[0]['texptime']
             self.expstart = plist[0]['texpstart']
-            self.expend = plist[_nimgs-1]['texpend']
+            self.expend = plist[_nimgs - 1]['texpend']
 
 
         if blot:
@@ -159,13 +165,13 @@ class OutputImage:
         self.outweight = _outweight
         self.outcontext = _outcontext
 
-    def set_bunit(self,bunit):
+    def set_bunit(self, bunit):
         """
         Method used to update the value of the bunit attribute.
         """
         self.bunit = bunit
 
-    def set_units(self,units):
+    def set_units(self, units):
         """
         Method used to record what units were specified by the user for the output product.
         """
@@ -173,7 +179,7 @@ class OutputImage:
 
     def writeFITS(self, template, sciarr, whtarr, ctxarr=None,
                 versions=None, overwrite=yes, blend=True, virtual=False,
-                rules_file=None):
+                rules_file=None, logfile=None):
         """
         Generate PyFITS objects for each output extension
         using the file given by 'template' for populating
@@ -234,19 +240,18 @@ class OutputImage:
         # If only writing out single drizzle product, blending needs to be
         # forced off as there is only 1 input to report, no blending needed
         if self.single:
-            blend=False
+            blend = False
 
         # If input data is not in MEF FITS format, it will return 'None'
         # and those headers will have to be generated from drizzle output
         # file FITS headers.
         # NOTE: These are HEADER objects, not HDUs
-        #prihdr,scihdr,errhdr,dqhdr = getTemplates(template)
         self.fullhdrs, intab = getTemplates(template, blend=False,
                                             rules_file=rules_file)
 
-        newhdrs, newtab = getTemplates(template,blend=blend,
+        newhdrs, newtab = getTemplates(template, blend=blend,
                                         rules_file=rules_file)
-        if newtab is not None: nextend += 1 # account for new table extn
+        if newtab is not None: nextend += 1  # account for new table extn
 
         prihdr = newhdrs[0]
         scihdr = newhdrs[1]
@@ -260,7 +265,8 @@ class OutputImage:
         prihdu.header.set('EXTEND', value=True, after='NAXIS')
         prihdu.header['NEXTEND'] = nextend
         prihdu.header['FILENAME'] = self.output
-        prihdu.header['PROD_VER'] = 'DrizzlePac {}'.format(version.__version__)
+        prihdu.header['PROD_VER'] = 'DrizzlePac {}'.format(__version__)
+        prihdu.header['DRIZPARS'] = (logfile, "Logfile for processing")
 
         # Update the ROOTNAME with the new value as well
         _indx = self.output.find('_drz')
@@ -280,7 +286,7 @@ class OutputImage:
             prihdu.header['EXPSTART'] = self.expstart
             prihdu.header['EXPEND'] = self.expend
 
-        #Update ASN_MTYPE to reflect the fact that this is a product
+        # Update ASN_MTYPE to reflect the fact that this is a product
         # Currently hard-wired to always output 'PROD-DTH' as MTYPE
         prihdu.header['ASN_MTYP'] = 'PROD-DTH'
 
@@ -291,27 +297,24 @@ class OutputImage:
         if 'DITHCORR' in prihdu.header:
             prihdu.header['DITHCORR'] = 'COMPLETE'
 
-        prihdu.header['NDRIZIM'] =(len(self.parlist),
+        prihdu.header['NDRIZIM'] = (len(self.parlist),
                                    'Drizzle, No. images drizzled onto output')
 
         # Only a subset of these keywords makes sense for the new WCS based
         # transformations. They need to be reviewed to decide what to keep
         # and what to leave out.
         if not self.blot:
-            self.addDrizKeywords(prihdu.header,versions)
+            self.addDrizKeywords(prihdu.header, versions)
 
         if scihdr:
-            try:
-                del scihdr['OBJECT']
-            except KeyError:
-                pass
+            scihdr.pop('OBJECT', None)
 
             if 'CCDCHIP' in scihdr: scihdr['CCDCHIP'] = '-999'
             if 'NCOMBINE' in scihdr:
                 scihdr['NCOMBINE'] = self.parlist[0]['nimages']
 
             # If BUNIT keyword was found and reset, then
-            bunit_last_kw = self.find_kwupdate_location(scihdr,'bunit')
+            bunit_last_kw = self.find_kwupdate_location(scihdr, 'bunit')
             if self.bunit is not None:
                 comment_str = "Units of science product"
                 if self.bunit.lower()[:5] == 'count':
@@ -329,17 +332,17 @@ class OutputImage:
 
             # Add WCS keywords to SCI header
             if self.wcs:
-                pre_wcs_kw = self.find_kwupdate_location(scihdr,'CD1_1')
-                addWCSKeywords(self.wcs,scihdr,blot=self.blot,
+                pre_wcs_kw = self.find_kwupdate_location(scihdr, 'CD1_1')
+                addWCSKeywords(self.wcs, scihdr, blot=self.blot,
                                 single=self.single, after=pre_wcs_kw)
                 # Recompute this after removing distortion kws
-                pre_wcs_kw = self.find_kwupdate_location(scihdr,'CD1_1')
+                pre_wcs_kw = self.find_kwupdate_location(scihdr, 'CD1_1')
 
         ##########
         # Now, build the output file
         ##########
         if self.build:
-            print('-Generating multi-extension output file: ',self.output)
+            print('-Generating multi-extension output file: ', self.output)
             fo = fits.HDUList()
 
             # Add primary header to output file...
@@ -349,7 +352,7 @@ class OutputImage:
                 hdu = fits.CompImageHDU(data=sciarr, header=scihdr, name=EXTLIST[0])
             else:
                 hdu = fits.ImageHDU(data=sciarr, header=scihdr, name=EXTLIST[0])
-            last_kw = self.find_kwupdate_location(scihdr,'EXTNAME')
+            last_kw = self.find_kwupdate_location(scihdr, 'EXTNAME')
             hdu.header.set('EXTNAME', value='SCI', after=last_kw)
             hdu.header.set('EXTVER', value=1, after='EXTNAME')
             fo.append(hdu)
@@ -362,14 +365,14 @@ class OutputImage:
                 hdu = fits.CompImageHDU(data=whtarr, header=errhdr, name=EXTLIST[1])
             else:
                 hdu = fits.ImageHDU(data=whtarr, header=errhdr, name=EXTLIST[1])
-            last_kw = self.find_kwupdate_location(errhdr,'EXTNAME')
+            last_kw = self.find_kwupdate_location(errhdr, 'EXTNAME')
             hdu.header.set('EXTNAME', value='WHT', after=last_kw)
             hdu.header.set('EXTVER', value=1, after='EXTNAME')
             if self.wcs:
-                pre_wcs_kw = self.find_kwupdate_location(hdu.header,'CD1_1')
+                pre_wcs_kw = self.find_kwupdate_location(hdu.header, 'CD1_1')
                 # Update WCS Keywords based on PyDrizzle product's value
                 # since 'drizzle' itself doesn't update that keyword.
-                addWCSKeywords(self.wcs,hdu.header,blot=self.blot,
+                addWCSKeywords(self.wcs, hdu.header, blot=self.blot,
                                single=self.single, after=pre_wcs_kw)
             fo.append(hdu)
 
@@ -387,34 +390,34 @@ class OutputImage:
                 hdu = fits.CompImageHDU(data=_ctxarr, header=dqhdr, name=EXTLIST[2])
             else:
                 hdu = fits.ImageHDU(data=_ctxarr, header=dqhdr, name=EXTLIST[2])
-            last_kw = self.find_kwupdate_location(dqhdr,'EXTNAME')
+            last_kw = self.find_kwupdate_location(dqhdr, 'EXTNAME')
             hdu.header.set('EXTNAME', value='CTX', after=last_kw)
             hdu.header.set('EXTVER', value=1, after='EXTNAME')
 
             if self.wcs:
-                pre_wcs_kw = self.find_kwupdate_location(hdu.header,'CD1_1')
+                pre_wcs_kw = self.find_kwupdate_location(hdu.header, 'CD1_1')
                 # Update WCS Keywords based on PyDrizzle product's value
                 # since 'drizzle' itself doesn't update that keyword.
-                addWCSKeywords(self.wcs,hdu.header,blot=self.blot,
+                addWCSKeywords(self.wcs, hdu.header, blot=self.blot,
                                single=self.single, after=pre_wcs_kw)
             fo.append(hdu)
 
             # remove all alternate WCS solutions from headers of this product
-            wcs_functions.removeAllAltWCS(fo,[1])
+            wcs_functions.removeAllAltWCS(fo, [1])
 
             # add table of combined header keyword values to FITS file
             if newtab is not None:
                 fo.append(newtab)
 
             if not virtual:
-                print('Writing out to disk:',self.output)
+                print('Writing out to disk:', self.output)
                 # write out file to disk
                 fo.writeto(self.output)
                 fo.close()
                 del fo, hdu
                 fo = None
             # End 'if not virtual'
-            outputFITS[self.output]= fo
+            outputFITS[self.output] = fo
 
         else:
             print('-Generating simple FITS output: %s' % self.outdata)
@@ -429,23 +432,27 @@ class OutputImage:
                 for _card in scihdr.cards:
                     if _card.keyword not in RESERVED_KEYS and _card.keyword not in hdu_header:
                         hdu_header.append(_card)
-            for kw in ['PCOUNT', 'GCOUNT']:
-                try:
-                    del kw
-                except KeyError:
-                    pass
+
             hdu_header['filename'] = self.outdata
 
             if self.compress:
                 hdu = fits.CompImageHDU(data=sciarr, header=hdu_header)
                 wcs_ext = [1]
             else:
-                hdu = fits.ImageHDU(data=sciarr, header=hdu_header)
+                hdu = fits.PrimaryHDU(data=sciarr, header=hdu_header)
                 wcs_ext = [0]
 
             # explicitly set EXTEND to FALSE for simple FITS files.
             dim = len(sciarr.shape)
-            hdu.header.set('extend', value=False, after='NAXIS%s'%dim)
+            hdu.header.set('extend', value=False, after='NAXIS%s' % dim)
+
+            # explicitly remove EXTNAME, EXTVER from header
+            # since this header may have been used
+            # to create a CompImageHDU instance instead of a PrimaryHDU instance
+            for kw in ['EXTNAME', 'EXTVER', 'PCOUNT', 'GCOUNT']:
+                hdu.header.pop(kw, None)
+
+            hdu.header.set('filetype', 'SCI', before='TELESCOP', comment='Type of data in array')
 
             # Add primary header to output file...
             fo.append(hdu)
@@ -453,7 +460,7 @@ class OutputImage:
             if not self.blot:
                 # remove all alternate WCS solutions from headers of this product
                 logutil.logging.disable(logutil.logging.INFO)
-                wcs_functions.removeAllAltWCS(fo,wcs_ext)
+                wcs_functions.removeAllAltWCS(fo, wcs_ext)
                 logutil.logging.disable(logutil.logging.NOTSET)
 
             # add table of combined header keyword values to FITS file
@@ -461,15 +468,15 @@ class OutputImage:
                 fo.append(newtab)
 
             if not virtual or "single_sci" in self.outdata:
-                print('Writing out image to disk:',self.outdata)
+                print('Writing out image to disk:', self.outdata)
                 # write out file to disk
-                fo.writeto(self.outdata)
+                fo.writeto(self.outdata, overwrite=True)
                 del hdu
                 if "single_sci" not in self.outdata:
                     del fo
                     fo = None
             # End 'if not virtual'
-            outputFITS[self.outdata]= fo
+            outputFITS[self.outdata] = fo
 
             if self.outweight and whtarr is not None:
                 # We need to build new PyFITS objects for each WHT array
@@ -481,7 +488,7 @@ class OutputImage:
                 if self.compress:
                     hdu = fits.CompImageHDU(data=whtarr, header=prihdu.header)
                 else:
-                    hdu = fits.ImageHDU(data=whtarr, header=prihdu.header)
+                    hdu = fits.PrimaryHDU(data=whtarr, header=prihdu.header)
                 # Append remaining unique header keywords from template DQ
                 # header to Primary header...
                 if errhdr:
@@ -490,25 +497,34 @@ class OutputImage:
                             hdu.header.append(_card)
                 hdu.header['filename'] = self.outweight
                 hdu.header['CCDCHIP'] = '-999'
+
                 if self.wcs:
-                    pre_wcs_kw = self.find_kwupdate_location(hdu.header,'CD1_1')
+                    pre_wcs_kw = self.find_kwupdate_location(hdu.header, 'CD1_1')
                     # Update WCS Keywords based on PyDrizzle product's value
                     # since 'drizzle' itself doesn't update that keyword.
-                    addWCSKeywords(self.wcs,hdu.header, blot=self.blot,
+                    addWCSKeywords(self.wcs, hdu.header, blot=self.blot,
                                    single=self.single, after=pre_wcs_kw)
 
+                # explicitly remove EXTNAME, EXTVER from header
+                # since this header may have been used
+                # to create a CompImageHDU instance instead of a PrimaryHDU instance
+                for kw in ['EXTNAME', 'EXTVER', 'PCOUNT', 'GCOUNT']:
+                    hdu.header.pop(kw, None)
+
+                hdu.header.set('filetype', 'WHT', before='TELESCOP', comment='Type of data in array')
+
                 # Add primary header to output file...
                 fwht.append(hdu)
                 # remove all alternate WCS solutions from headers of this product
-                wcs_functions.removeAllAltWCS(fwht,wcs_ext)
+                wcs_functions.removeAllAltWCS(fwht, wcs_ext)
 
                 if not virtual:
-                    print('Writing out image to disk:',self.outweight)
-                    fwht.writeto(self.outweight)
-                    del fwht,hdu
+                    print('Writing out image to disk:', self.outweight)
+                    fwht.writeto(self.outweight, overwrite=True)
+                    del fwht, hdu
                     fwht = None
                 # End 'if not virtual'
-                outputFITS[self.outweight]= fwht
+                outputFITS[self.outweight] = fwht
 
             # If a context image was specified, build a PyFITS object
             # for it as well...
@@ -524,37 +540,46 @@ class OutputImage:
                 if self.compress:
                     hdu = fits.CompImageHDU(data=_ctxarr, header=prihdu.header)
                 else:
-                    hdu = fits.ImageHDU(data=_ctxarr, header=prihdu.header)
+                    hdu = fits.PrimaryHDU(data=_ctxarr, header=prihdu.header)
                 # Append remaining unique header keywords from template DQ
                 # header to Primary header...
                 if dqhdr:
                     for _card in dqhdr.cards:
-                        if ( (_card.keyword not in RESERVED_KEYS) and
+                        if ((_card.keyword not in RESERVED_KEYS) and
                              _card.keyword not in hdu.header):
                             hdu.header.append(_card)
                 hdu.header['filename'] = self.outcontext
+
                 if self.wcs:
-                    pre_wcs_kw = self.find_kwupdate_location(hdu.header,'CD1_1')
+                    pre_wcs_kw = self.find_kwupdate_location(hdu.header, 'CD1_1')
                     # Update WCS Keywords based on PyDrizzle product's value
                     # since 'drizzle' itself doesn't update that keyword.
-                    addWCSKeywords(self.wcs,hdu.header, blot=self.blot,
+                    addWCSKeywords(self.wcs, hdu.header, blot=self.blot,
                                    single=self.single, after=pre_wcs_kw)
 
+                # explicitly remove EXTNAME, EXTVER from header
+                # since this header may have been used
+                # to create a CompImageHDU instance instead of a PrimaryHDU instance
+                for kw in ['EXTNAME', 'EXTVER', 'PCOUNT', 'GCOUNT']:
+                    hdu.header.pop(kw, None)
+
+                hdu.header.set('filetype', 'CTX', before='TELESCOP', comment='Type of data in array')
+
                 fctx.append(hdu)
                 # remove all alternate WCS solutions from headers of this product
-                wcs_functions.removeAllAltWCS(fctx,wcs_ext)
+                wcs_functions.removeAllAltWCS(fctx, wcs_ext)
                 if not virtual:
-                    print('Writing out image to disk:',self.outcontext)
-                    fctx.writeto(self.outcontext)
-                    del fctx,hdu
+                    print('Writing out image to disk:', self.outcontext)
+                    fctx.writeto(self.outcontext, overwrite=True)
+                    del fctx, hdu
                     fctx = None
                 # End 'if not virtual'
 
-                outputFITS[self.outcontext]= fctx
+                outputFITS[self.outcontext] = fctx
 
         return outputFITS
 
-    def find_kwupdate_location(self,hdr,keyword):
+    def find_kwupdate_location(self, hdr, keyword):
         """
         Find the last keyword in the output header that comes before the new
         keyword in the original, full input headers.
@@ -567,7 +592,6 @@ class OutputImage:
         last_kw = None
         for extn in self.fullhdrs:
             if keyword in extn:
-                #indx = extn.ascard.index_of(keyword)
                 indx = extn.index(keyword)
                 kw_list = list(extn.keys())[:indx]
                 break
@@ -589,7 +613,7 @@ class OutputImage:
         return last_kw
 
 
-    def addDrizKeywords(self,hdr,versions):
+    def addDrizKeywords(self, hdr, versions):
         """ Add drizzle parameter keywords to header. """
 
         # Extract some global information for the keywords
@@ -600,25 +624,27 @@ class OutputImage:
 
             # Start by building up the keyword prefix based
             # on the image number for the chip
-            #_keyprefix = 'D%03d'%_imgnum
+            # _keyprefix = 'D%03d'%_imgnum
             _imgnum += 1
 
             drizdict = DRIZ_KEYWORDS.copy()
             # Update drizdict with current values
-            drizdict['VER']['value'] = pl['driz_version'][:44]
-            drizdict['DATA']['value'] = pl['data'][:64]
+            # Any limit on the length of the strings was removed as an update to
+            # new versions of the FITS standard and to accommodate MVM processing.
+            drizdict['VER']['value'] = pl['driz_version']
+            drizdict['DATA']['value'] = pl['data']
             drizdict['DEXP']['value'] = pl['exptime']
-            drizdict['OUDA']['value'] = pl['outFinal'][:64]
-            drizdict['OUWE']['value'] = pl['outWeight'][:64]
+            drizdict['OUDA']['value'] = pl['outFinal']
+            drizdict['OUWE']['value'] = pl['outWeight']
             if pl['outContext'] is None:
                 outcontext = ""
             else:
-                outcontext = pl['outContext'][:64]
+                outcontext = pl['outContext']
             drizdict['OUCO']['value'] = outcontext
             if self.single:
-                drizdict['MASK']['value'] = pl['singleDrizMask'][:64]
+                drizdict['MASK']['value'] = pl['singleDrizMask']
             else:
-                drizdict['MASK']['value'] = pl['finalMask'][:64]
+                drizdict['MASK']['value'] = pl['finalMask']
 
             # Process the values of WT_SCL to be consistent with
             # what IRAF Drizzle would output
@@ -626,7 +652,7 @@ class OutputImage:
                 _wtscl = pl['wt_scl_val']
             else:
                 if pl['wt_scl'] == 'exptime': _wtscl = pl['exptime']
-                elif pl['wt_scl'] == 'expsq': _wtscl = pl['exptime']*pl['exptime']
+                elif pl['wt_scl'] == 'expsq': _wtscl = pl['exptime'] * pl['exptime']
                 else: _wtscl = pl['wt_scl']
 
             drizdict['WTSC']['value'] = _wtscl
@@ -640,11 +666,11 @@ class OutputImage:
             drizdict['FVAL']['value'] = _fillval
             drizdict['WKEY']['value'] = pl['driz_wcskey']
 
-            drizdict['SCAL'] = {'value':pl['scale'],'comment':'Drizzle, pixel size (arcsec) of output image'}
-            drizdict['ISCL'] = {'value':pl['idcscale'],'comment':'Drizzle, default IDCTAB pixel size(arcsec)'}
+            drizdict['SCAL'] = {'value': pl['scale'], 'comment': 'Drizzle, pixel size (arcsec) of output image'}
+            drizdict['ISCL'] = {'value': pl['idcscale'], 'comment': 'Drizzle, default IDCTAB pixel size(arcsec)'}
 
             # Now update header with values
-            writeDrizKeywords(hdr,_imgnum,drizdict)
+            writeDrizKeywords(hdr, _imgnum, drizdict)
             del drizdict
 
         # Add version information as HISTORY cards to the header
@@ -652,26 +678,17 @@ class OutputImage:
             ver_str = "AstroDrizzle processing performed using: "
             hdr.add_history(ver_str)
             for k in versions.keys():
-                ver_str = '    '+str(k)+' Version '+str(versions[k])
+                ver_str = '    ' + str(k) + ' Version ' + str(versions[k])
                 hdr.add_history(ver_str)
 
 
-def cleanTemplates(scihdr,errhdr,dqhdr):
+def cleanTemplates(scihdr, errhdr, dqhdr):
 
     # Now, safeguard against having BSCALE and BZERO
     for kw in ['BSCALE', 'BZERO']:
-        try:
-            del scihdr[kw]
-        except KeyError:
-            pass
-        try:
-            del errhdr[kw]
-        except KeyError:
-            pass
-        try:
-            del dqhdr[kw]
-        except KeyError:
-            pass
+        scihdr.pop(kw, None)
+        errhdr.pop(kw, None)
+        dqhdr.pop(kw, None)
 
     # At this point, check errhdr and dqhdr to make sure they
     # have all the requisite keywords (as listed in updateDTHKeywords).
@@ -682,7 +699,8 @@ def cleanTemplates(scihdr,errhdr,dqhdr):
                 if keyword not in errhdr:
                     errhdr[keyword] = scihdr[keyword]
                 if keyword not in dqhdr:
-                    dqhdr[keyword]= scihdr[keyword]
+                    dqhdr[keyword] = scihdr[keyword]
+
 
 def getTemplates(fnames, blend=True, rules_file=None):
     """ Process all headers to produce a set of combined headers
@@ -690,7 +708,7 @@ def getTemplates(fnames, blend=True, rules_file=None):
 
     """
     if not blend:
-        newhdrs =  blendheaders.getSingleTemplate(fnames[0])
+        newhdrs = blendheaders.getSingleTemplate(fnames[0])
         newtab = None
     else:
         # apply rules to create final version of headers, plus table
@@ -699,11 +717,11 @@ def getTemplates(fnames, blend=True, rules_file=None):
         newhdrs, newtab = blendheaders.get_blended_headers(inputs=fnames,
                                                             rules_file=rules_file)
 
-    cleanTemplates(newhdrs[1],newhdrs[2],newhdrs[3])
+    cleanTemplates(newhdrs[1], newhdrs[2], newhdrs[3])
 
     return newhdrs, newtab
 
-def addWCSKeywords(wcs,hdr,blot=False,single=False,after=None):
+def addWCSKeywords(wcs, hdr, blot=False, single=False, after=None):
     """ Update input header 'hdr' with WCS keywords.
     """
     wname = wcs.wcs.name
@@ -743,20 +761,24 @@ def deleteDistortionKeywords(hdr):
     """ Delete distortion related keywords from output drizzle science header
         since the drizzled image should have no remaining distortion.
     """
-    dist_kws = ['D2IMERR1','D2IMERR2','D2IMDIS1','D2IMDIS2','D2IM1.*','D2IM2.*',
-                'D2IMEXT', 'DP1', 'DP2', 'CPDIS?', 'CPERR?']
-    for kw in dist_kws:
-        if kw in hdr:
-            del hdr[kw]
+    # We need to use '.pop' to guard against the possibility, however remote,
+    # that the keyword has already been removed before calling this function.
+    for kw in DIST_KWS:
+        hdr.pop(kw, None)
+
+    # This can use 'del' since it will work even if the keywords
+    # are missing altogether since the multi_kw uses wild-cards
+    for multi_kw in DIST_MULTI_KWS:
+        del hdr[multi_kw]
 
 
-def writeSingleFITS(data,wcs,output,template,clobber=True,verbose=True,
+def writeSingleFITS(data, wcs, output, template, clobber=True, verbose=True,
                     rules_file=None):
     """ Write out a simple FITS file given a numpy array and the name of another
     FITS file to use as a template for the output image header.
     """
-    outname,outextn = fileutil.parseFilename(output)
-    outextname,outextver = fileutil.parseExtn(outextn)
+    outname, outextn = fileutil.parseFilename(output)
+    outextname, outextver = fileutil.parseExtn(outextn)
 
     if fileutil.findFile(outname):
         if clobber:
@@ -770,7 +792,7 @@ def writeSingleFITS(data,wcs,output,template,clobber=True,verbose=True,
             raise IOError
 
     # Now update WCS keywords with values from provided WCS
-    if hasattr(wcs.sip,'a_order'):
+    if hasattr(wcs.sip, 'a_order'):
         siphdr = True
     else:
         siphdr = False
@@ -780,16 +802,16 @@ def writeSingleFITS(data,wcs,output,template,clobber=True,verbose=True,
         # Get default headers from multi-extension FITS file
         # If input data is not in MEF FITS format, it will return 'None'
         # NOTE: These are HEADER objects, not HDUs
-        (prihdr,scihdr,errhdr,dqhdr),newtab = getTemplates(template,EXTLIST,
+        (prihdr, scihdr, errhdr, dqhdr), newtab = getTemplates(template, EXTLIST,
                                                            rules_file=rules_file)
 
         if scihdr is None:
             scihdr = fits.Header()
             indx = 0
             for c in prihdr.cards:
-                if c.keyword not in ['INHERIT','EXPNAME']: indx += 1
+                if c.keyword not in ['INHERIT', 'EXPNAME']: indx += 1
                 else: break
-            for i in range(indx,len(prihdr)):
+            for i in range(indx, len(prihdr)):
                 scihdr.append(prihdr.cards[i])
             for i in range(indx, len(prihdr)):
                 del prihdr[indx]
@@ -811,7 +833,7 @@ def writeSingleFITS(data,wcs,output,template,clobber=True,verbose=True,
     outhdu = fits.HDUList()
     # Setup primary header as an HDU ready for appending to output FITS file
     prihdu = fits.PrimaryHDU(header=prihdr)
-    scihdu = fits.ImageHDU(header=scihdr,data=data)
+    scihdu = fits.ImageHDU(header=scihdr, data=data)
 
     outhdu.append(prihdu)
     outhdu.append(scihdu)
@@ -820,18 +842,18 @@ def writeSingleFITS(data,wcs,output,template,clobber=True,verbose=True,
     if verbose:
         print('Created output image: %s' % outname)
 
-def writeDrizKeywords(hdr,imgnum,drizdict):
+def writeDrizKeywords(hdr, imgnum, drizdict):
     """ Write basic drizzle-related keywords out to image header as a record
         of the processing performed to create the image
 
         The dictionary 'drizdict' will contain the keywords and values to be
         written out to the header.
     """
-    _keyprefix = 'D%03d'%imgnum
+    _keyprefix = 'D%03d' % imgnum
 
     for key in drizdict:
         val = drizdict[key]['value']
         if val is None: val = ""
         comment = drizdict[key]['comment']
         if comment is None: comment = ""
-        hdr[_keyprefix+key] = (val, drizdict[key]['comment'])
+        hdr[_keyprefix + key] = (val, drizdict[key]['comment'])
