@@ -12,9 +12,9 @@ input image while recording the subtracted value in the image header.
 import os, sys
 
 import numpy as np
-from astropy.nddata import interpret_bit_flags
 
 from stsci.tools import fileutil, teal, logutil
+from stsci.tools.bitmask import interpret_bit_flags
 import stsci.imagestats as imagestats
 
 from stsci.skypac.skymatch import skymatch
@@ -26,7 +26,7 @@ from . import processInput
 from .imageObject import imageObject
 
 from . import util
-from .version import *
+from . import __version__
 
 
 __taskname__= "drizzlepac.sky" #looks in drizzlepac for sky.cfg
@@ -212,8 +212,8 @@ def _skymatch(imageList, paramDict, in_memory, clean, logfile):
 
     nimg = len(imageList)
     if nimg == 0:
-        ml.logentry("Skymatch needs at least one images to perform{0}" \
-                    "sky matching. Nothing to be done.",os.linesep)
+        log.info("Skymatch needs at least one image to perform{0} \
+                    sky matching. Nothing to be done.",os.linesep)
         return
 
     # create a list of input file names as provided by the user:
@@ -327,26 +327,54 @@ def _skymatch(imageList, paramDict, in_memory, clean, logfile):
 
         new_fi.append(fi)
 
-    # Run skymatch algorithm:
-    skymatch(new_fi,
-             skymethod   = paramDict['skymethod'],
-             skystat     = paramDict['skystat'],
-             lower       = paramDict['skylower'],
-             upper       = paramDict['skyupper'],
-             nclip       = paramDict['skyclip'],
-             lsigma      = paramDict['skylsigma'],
-             usigma      = paramDict['skyusigma'],
-             binwidth    = paramDict['skywidth'],
-             skyuser_kwd = skyKW,
-             units_kwd   = 'BUNIT',
-             readonly    = not paramDict['skysub'],
-             dq_bits     = None,
-             optimize    = 'inmemory' if in_memory else 'balanced',
-             clobber     = True,
-             clean       = clean,
-             verbose     = True,
-             flog        = MultiFileLog(console = True, enableBold = False),
-             _taskname4history = 'AstroDrizzle')
+    try:
+        # Run skymatch algorithm:
+        skymatch(new_fi,
+                 skymethod   = paramDict['skymethod'],
+                 skystat     = paramDict['skystat'],
+                 lower       = paramDict['skylower'],
+                 upper       = paramDict['skyupper'],
+                 nclip       = paramDict['skyclip'],
+                 lsigma      = paramDict['skylsigma'],
+                 usigma      = paramDict['skyusigma'],
+                 binwidth    = paramDict['skywidth'],
+                 skyuser_kwd = skyKW,
+                 units_kwd   = 'BUNIT',
+                 readonly    = not paramDict['skysub'],
+                 dq_bits     = None,
+                 optimize    = 'inmemory' if in_memory else 'balanced',
+                 clobber     = True,
+                 clean       = clean,
+                 verbose     = True,
+                 flog        = MultiFileLog(console = True, enableBold = False),
+                 _taskname4history = 'AstroDrizzle')
+    except AssertionError:
+        if 'match' in paramDict['skymethod']:  # This catches 'match' and 'globalmin+match'
+            new_method = 'globalmin' if 'globalmin' in paramDict['skymethod'] else 'localmin'
+
+            # revert to simpler sky computation algorithm
+            log.warning('Reverting sky computation to "localmin" from "{}'.format(paramDict['skymethod']))
+            skymatch(new_fi,
+                     skymethod=new_method,
+                     skystat=paramDict['skystat'],
+                     lower=paramDict['skylower'],
+                     upper=paramDict['skyupper'],
+                     nclip=paramDict['skyclip'],
+                     lsigma=paramDict['skylsigma'],
+                     usigma=paramDict['skyusigma'],
+                     binwidth=paramDict['skywidth'],
+                     skyuser_kwd=skyKW,
+                     units_kwd='BUNIT',
+                     readonly=not paramDict['skysub'],
+                     dq_bits=None,
+                     optimize='inmemory' if in_memory else 'balanced',
+                     clobber=True,
+                     clean=clean,
+                     verbose=True,
+                     flog=MultiFileLog(console=True, enableBold=False),
+                     _taskname4history='AstroDrizzle')
+        else:
+            raise
 
     # Populate 'subtractedSky' and 'computedSky' of input image objects:
     for i in range(nimg):
@@ -795,58 +823,10 @@ def getreferencesky(image,keyval):
     _refplatescale=image.header["REFPLTSCL"]
     _platescale=image.header["PLATESCL"]
 
-    return (_subtractedsky * (_refplatescale / _platescale)**2 )
+    return (_subtractedSky * (_refplatescale / _platescale)**2 )
 
 
-def help(file=None):
-    """
-    Print out syntax help for running astrodrizzle
+sky.__doc__ = util._def_help_functions(
+    locals(), module_file=__file__, task_name=__taskname__, module_doc=__doc__
+)
 
-    Parameters
-    ----------
-    file : str (Default = None)
-        If given, write out help to the filename specified by this parameter
-        Any previously existing file with this name will be deleted before
-        writing out the help.
-
-    """
-    helpstr = getHelpAsString(docstring=True, show_ver = True)
-    if file is None:
-        print(helpstr)
-    else:
-        if os.path.exists(file): os.remove(file)
-        f = open(file, mode = 'w')
-        f.write(helpstr)
-        f.close()
-
-
-def getHelpAsString(docstring = False, show_ver = True):
-    """
-    return useful help from a file in the script directory called
-    __taskname__.help
-
-    """
-    install_dir = os.path.dirname(__file__)
-    taskname = util.base_taskname(__taskname__, __package__)
-    htmlfile = os.path.join(install_dir, 'htmlhelp', taskname + '.html')
-    helpfile = os.path.join(install_dir, taskname + '.help')
-
-    if docstring or (not docstring and not os.path.exists(htmlfile)):
-        if show_ver:
-            helpString = os.linesep + \
-                ' '.join([__taskname__, 'Version', __version__,
-                ' updated on ', __version_date__]) + 2*os.linesep
-        else:
-            helpString = ''
-        if os.path.exists(helpfile):
-            helpString += teal.getHelpFileAsString(taskname, __file__)
-        else:
-            if __doc__ is not None:
-                helpString += __doc__ + os.linesep
-    else:
-        helpString = 'file://' + htmlfile
-
-    return helpString
-
-
-sky.__doc__ = getHelpAsString(docstring = True, show_ver = False)
